/*
 * GUTS - Generic Utilities 
 * Copyright (c) 2006-2010 Gino van den Bergen, DTECTA 
 * http://www.dtecta.com
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Gino van den Bergen makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 */

#ifndef GUTS_INVARIANT_HPP
#define GUTS_INVARIANT_HPP

#ifndef NDEBUG
//#   define PARANOID
#endif 

// A bug in VC7.1 and 8.0 does not allow me to declare friend class templates from a different namespace.
#if defined (_MSC_VER) 
#   define INVARIANT public: void _invariant_() const
#else 
#   define INVARIANT template <typename> friend class ::guts::InvariantGuard; void _invariant_() const
#endif

#ifdef PARANOID
#   define GUARD_INVARIANT(ptr) const guts::Guard& _guard_ = guts::makeInvariantGuard(ptr); ((void)_guard_)
#   define GUARD_INVARIANT_AT_EXIT(ptr) const guts::Guard& _guard_ = guts::makeInvariantGuardExit(ptr); ((void)_guard_)
#else
#   define GUARD_INVARIANT(ptr) ((void)0)
#   define GUARD_INVARIANT_AT_EXIT(ptr) ((void)0)
#endif

namespace guts
{
    class Guard {};

    template <typename T>
    class InvariantGuard
        : public Guard
    {
    public: 
        InvariantGuard(const T* ptr, int)
            : m_ptr(ptr)
        {}

        InvariantGuard(const T* ptr)
            : m_ptr(ptr)
        {
            m_ptr->_invariant_();
        }

        ~InvariantGuard()
        {
            m_ptr->_invariant_();   
        }

    private:
        const T* m_ptr;
    };

    template <typename T>
    InvariantGuard<T> makeInvariantGuard(T* ptr)
    {
        return InvariantGuard<T>(ptr);
    } 

    template <typename T>
    InvariantGuard<T> makeInvariantGuardExit(T* ptr)
    {
        return InvariantGuard<T>(ptr, 0);
    }
}

#endif
