/*
 * GUTS - Generic Utilities 
 * Copyright (c) 2006-2010 Gino van den Bergen, DTECTA 
 * http://www.dtecta.com
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Gino van den Bergen makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 */

#ifndef GUTS_LIST_HPP
#define GUTS_LIST_HPP

#include <consolid/consolid.h>

namespace guts
{
    class Link
    {
        friend class List;
        
    public:
        Link()
        {
            m_prev = m_next = this;
        }
  
        Link* next() const
        {
            return m_next;
        } 
        
        Link* prev() const
        {
            return m_prev;
        }
        
        void insert(Link* link)
        {
            m_next = link;
            m_prev = link->m_prev;
            m_next->m_prev = this; 
            m_prev->m_next = this;
        } 

        void remove()
        {
            m_next->m_prev = m_prev;
            m_prev->m_next = m_next;
            m_next = m_prev = this;
        }

        bool isFree() const 
        {
            return m_next == this;
        }

    private: 
        Link* m_next;
        Link* m_prev;
    };


   
    class List
    {
    public: 
        Link* begin() const
        {
            return m_head.m_next;
        }

        Link* end() const 
        {
            return const_cast<Link*>(&m_head);
        }  
        
        Link* front() const
        {
            ASSERT(!empty());
            return m_head.m_next;
        }

        Link* back() const 
        {   
            ASSERT(!empty());
            return m_head.m_prev;
        } 

        bool empty() const
        {
            return m_head.m_next == &m_head;
        }

        void push_front(Link* link)
        {
            link->insert(m_head.m_next);
        }

        void push_back(Link* link)
        {
            link->insert(&m_head);
        }

        void clear()
        {
            while (!empty())
            {
                front()->remove();   
            }
        }

    private:
        Link m_head;
    };    
}


#endif
