/*
 * GUTS - Generic Utilities 
 * Copyright (c) 2006-2010 Gino van den Bergen, DTECTA 
 * http://www.dtecta.com
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Gino van den Bergen makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 */

#ifndef GUTS_SHAREDPTR_HPP
#define GUTS_SHAREDPTR_HPP

#include "List.hpp"

namespace guts
{
    template <typename T>
    class SharedPtr
    {
    public:
        SharedPtr(T* ptr = 0)
            : m_ptr(ptr)
        {}

        SharedPtr(const SharedPtr<T>& rhs)
        {
            acquire(rhs);
        }

        ~SharedPtr()
        {
            release();  
        }

        SharedPtr<T>& operator=(const SharedPtr<T>& rhs)
        {
            if (this != &rhs)
            {
                release();
                acquire(rhs);
            }
            return *this;
        }

        T& operator*() const { return *m_ptr; }
        T* operator->() const { return m_ptr; }
        T* get() const { return m_ptr; }

        void reset(T* ptr = 0)
        {
            if (m_ptr != ptr)
            {
                release();
                m_ptr = ptr;
            }
        }

    private:
        void acquire(const SharedPtr<T>& rhs)
        {
            m_ptr = rhs.m_ptr;
            if (rhs.m_ptr != 0)
            {
                m_link.insert(&rhs.m_link);
            }
        }
        
        void release()
        {
            if (m_link.next() == &m_link)
            {
                delete m_ptr;
            }
            else
            {
                m_link.remove();
            }  
        }

        T* m_ptr;
        mutable Link m_link;
    };
}

#endif
